@@ -138,6 +138,15 @@ class BaseRestructurer(abc.ABC):
         # Only stop the session if this class created it
         if self.spark and self._created_session:
             self.spark.stop()
+            logger.info("ending spark session")
+            try:
+                gateway = getattr(self.spark.sparkContext, "_gateway", None)
+                if gateway:
+                    gateway.shutdown()
+                    logger.info("Py4J gateway shut down successfully.")
+            except Exception as e:
+                logger.warning(f"Error shutting down Py4J gateway: {e}")
+
 
     def _get_spark_session(self) -> SparkSession:
         """Initializes and returns a Spark session."""
